"""The pytest fixtures
   https://docs.pytest.org/en/latest/fixture.html
"""
# pylint: disable=missing-function-docstring, unused-import, wildcard-import, unused-wildcard-import, line-too-long, unused-argument
import pytest
from icpp.conftest_base import *  # pytest fixtures provided by icpp

# Define your pytest fixtures below
