#pragma once

#include "wasm_symbol.h"

void greet_0() WASM_SYMBOL_EXPORTED("canister_query greet_0");
void greet_0_auth() WASM_SYMBOL_EXPORTED("canister_query greet_0_auth");
void greet_1() WASM_SYMBOL_EXPORTED("canister_query greet_1");
void greet_2() WASM_SYMBOL_EXPORTED("canister_query greet_2");
void greet_3() WASM_SYMBOL_EXPORTED("canister_query greet_3");
void greet_4() WASM_SYMBOL_EXPORTED("canister_query greet_4");
void greet_json() WASM_SYMBOL_EXPORTED("canister_query greet_json");
