"""
Defines the version for the compatible wasi-sdk compiler.

(-) Release scripts might modify this automatically...
(-) do not add anything but the version number of the wasi-sdk compiler here!

"""
__version__ = "wasi-sdk-20.0"
