// To be included in the `native/main.cpp` of icpp projects.
#pragma once

#include "candid_deserialize.h"
#include "candid_serialize.h"
#include "candid_type.h"
#include "candid_type_all_includes.h"
#include "global.h"